function raveupdatedataset(varargin)
%raveupdatedataset(src,ev,updatetype,skipthese,dataset)
%applies the update in updatetype to all graphs that share the same
%dataset as the active graph or the dataset arguement.
%options for updatetype:
%'preferences','targets','replaceall','addrows','addcolumns','removerows',
%'removecolumns','datachanged'
%
%Note: addrows does not reset selection/coloration etc. Replaceall resets those if number of rows changes. (really should prompt and ask what to do).
% columns just updates the maintable RaveTableModel
%
%Use "replaceall" to also replace the backupdata set. Use "datachanged" to
%keep the same backupdata set.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

src=varargin{1};
ev=varargin{2};
updatetype=varargin{3};
skipthese=varargin{4};

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

if numel(varargin)==5
    d=varargin{5};
else
    d=udata.view.dataset;
end

%update backupdata
backupdata=getappdata(handles.fig,'backupdata');
switch updatetype
    case {'replaceall','removerows'} %TO DO: make it so removerows just removes relevant rows from existing backupdata.
        backupdata{d}=handles.alldata{d};
    case {'addrows','addcolumns'}
        newbackupdata=handles.alldata{d};
        olddims=size(backupdata{d});
        newbackupdata(1:olddims(1),1:olddims(2))=backupdata{d};
        backupdata{d}=newbackupdata;

end
setappdata(handles.fig,'backupdata',backupdata);

%reset minlimit/maxlimit if replacing data set:
for a=1:numel(handles.analysisnames)
    if size(handles.alldata{d},1)>0 %if data set is empty, do not change limits
        switch updatetype
            case {'replaceall'}
                handles.minlimit{d}{a}=min(handles.alldata{d},[],1);
                handles.maxlimit{d}{a}=max(handles.alldata{d},[],1);
                %also reset points of interest:
                handles.ispointofinterest{d}=false(size(handles.ispointofinterest{d}));
            otherwise
                %only update if limits are bigger than before:
                handles.minlimit{d}{a}=min(handles.minlimit{d}{a},min(handles.alldata{d},[],1));
                handles.maxlimit{d}{a}=max(handles.maxlimit{d}{a},max(handles.alldata{d},[],1));
        end
        %if minfilter/maxfilter is now outside of minlimit/maxlimit, update to be
        %within new bounds:
        handles.minfilter{d}{a}=max(handles.minfilter{d}{a},handles.minlimit{d}{a});
        handles.maxfilter{d}{a}=min(handles.maxfilter{d}{a},handles.maxlimit{d}{a});
    end
end

%reevaluate constraints:
for a=1:numel(handles.analysisnames)
    for c=1:numel(handles.constrainthandles{d}{a})
        handles.constraintvalues{d}{a}{c}=raveevaluateconstraint(handles,d,a,c,handles.alldata{d},true(size(handles.alldata{d},1)));
    end
end

othergraphs=find(handles.graphdataset==d & ishandle(handles.allgraphs));
needsupdating=false(size(othergraphs));
%apply changes to data set:
for o=1:numel(othergraphs);
    if ismember(handles.graphinfo.(get(handles.allgraphs(othergraphs(o)),'userdata')).updates,updatetype)
        needsupdating(o)=true;
    end
end

switch updatetype
    case {'preferences','targets'}
        %re-evaluate objective functions that use these variables:
        
        objectives=find(handles.isobjective{d});
        
        %NOTE: this loop won't work if one objective fcn is a function of another objective fcn.
        %should really check for dependency and evaluate in correct order.
        if ~isempty(objectives)
            newvalues=raveevaluatefunction(src,d,objectives,handles.alldata{d});
            handles.alldata{d}(:,objectives)=newvalues;
        end
        
        %determine which graphs need updating:
        if ~isempty(objectives)
            for o=1:numel(othergraphs);
                udat=get(handles.fakeax(othergraphs(o)),'userdata');
                allvars=unique([udat.view.x(:);udat.view.y(:);udat.view.z(:);udat.view.c(:);udat.view.s(:)]);
                if any(ismember(allvars,objectives))
                    needsupdating(o)=true;
                end
            end
        end
        
    case {'replaceall','addrows','removerows','datachanged'}
        handles.tablemodelupdatespending{d}=union(handles.tablemodelupdatespending{d},'data');
        needsupdating=false(size(othergraphs));
        for o=1:numel(othergraphs);
            thistype=get(handles.allgraphs(othergraphs(o)),'userdata');
            if ismember(updatetype,handles.graphinfo.(thistype).updates);
                needsupdating(o)=true;
            end
        end
end

guidata(handles.fig,handles);

%update table model:
switch updatetype
    case 'addcolumns'
        a=1;
        mod=handles.maintable.models{d};
        oldcols=mod.getColumnCount;
        newcols=size(handles.alldata{d},2);
        n2add=newcols-oldcols;
        for j=1:n2add
            c=oldcols+j;
            data=handles.alldata{d}(:,c);
            datatypes=handles.datatype{d}{c};
            dblArray=num2cell(data);
            for c=1:size(data,2)
                switch datatypes
                    case 'string'
                        dblArray(:,c)=handles.discretevalues{d}{a}{c}(data(:,c));
                    case {'integer','logical'}
                        dblArray(:,c)=num2cell(int32(data(:,c)));
                    otherwise %no nothing, values are double by default.  
                end
            end
            cname=java.lang.String(handles.allnames{d}{oldcols+j});
            mod.addColumn(cname,dblArray)
        end
        guidata(handles.fig,handles);
end

if handles.tablemode
    drawnow; %avoid java crash
    ravedisplaymaintable(handles.fig,[]);
    drawnow;
end

%find indices of other graphs with this dataset and analysis:
othergraphs=othergraphs(needsupdating);

othergraphs=setdiff(othergraphs,skipthese);

if isempty(othergraphs);return;end
originalindex=handles.activeindex;

for ai=othergraphs
    handles.activeindex=ai;
    
    othergraph=handles.allgraphs(ai);
    if ~ishandle(othergraph);continue;end %skip if this graph has been deleted.
    thistype=get(othergraph,'userdata'); %type of other graph in this analysis
    
    handles.activetype=thistype;
    handles.activegraph=othergraph;
    guidata(handles.fig,handles);
    
    if ~isempty(which(['raveupdate',thistype,updatetype]))
        eval(['raveupdate',thistype,updatetype,'(handles.fig,[]);'])
    else %brute force method:
        eval(['ravecreate',thistype,'(handles.fig,[]);'])
    end
    handles=guidata(handles.fig); %refresh handles;

    if ai~=originalindex
        raverasterize(handles.fig,[],ai)
        handles=guidata(handles.fig);
    end
end

othergraph=handles.allgraphs(originalindex);
thistype=get(othergraph,'userdata'); %type of other graph in this analysis

handles.activeindex=originalindex;
handles.activetype=thistype;
handles.activegraph=othergraph;
guidata(handles.fig,handles);

